# $Id$

require "rubygems"
require "spec"
require "tmpdir"
require "fileutils"

require "#{File.dirname __FILE__}/../ext/senna/senna"

describe '' do
  before do
    @tmpdir = "#{Dir.tmpdir}/senna_spec.#{$$}"
    begin
      Dir.mkdir @tmpdir
    rescue Errno::EEXIST
      @tmpdir << "_"
    end
  end
  after do
    FileUtils.rm_rf @tmpdir
  end

  describe Senna::Index do
    describe '.create:' do
      describe '' do
        it '正常に作成される' do
          Senna::Index.create("#{@tmpdir}/hoge").should be_kind_of(Senna::Index)
        end
      end

      describe 'ディレクトリがない場合:' do
        it 'Senna::Error' do
          proc{Senna::Index.create "#{@tmpdir}/hoge/hoge"}.should raise_error(Senna::Error, "cannot create: #{@tmpdir}/hoge/hoge: No such file or directory")
        end
      end

      describe '既に存在する場合:' do
        it '上書きされる' do
          File.open("#{@tmpdir}/hoge.SEN.i", "w").close
          Senna::Index.create("#{@tmpdir}/hoge").should be_kind_of(Senna::Index)
        end
      end

      describe 'path に汚染文字列を指定:' do
        it 'SecurityError' do
          $SAFE = 1
          proc{Senna::Index.create("#{@tmpdir}/hoge".taint)}.should raise_error(SecurityError, 'Insecure operation - create')
        end
      end

      describe 'path に文字列以外を指定:' do
        it 'TypeError' do
          proc{Senna::Index.create(123)}.should raise_error(TypeError, 'wrong argument type Fixnum (expected String)')
        end
      end

      describe '第2パラメータに Hash 以外を指定:' do
        it 'TypeError' do
          proc{Senna::Index.create("#{@tmpdir}/hoge", 123)}.should raise_error(TypeError, 'wrong argument type Fixnum (expected Hash)')
        end
      end

      describe 'key_size を指定:' do
        it '正常' do
          Senna::Index.create("#{@tmpdir}/hoge", :key_size=>100).should be_kind_of(Senna::Index)
        end
      end

      describe 'key_size に Integer 以外を指定:' do
        it 'TypeError' do
          proc{Senna::Index.create("#{@tmpdir}/hoge", :key_size=>"123")}.should raise_error(TypeError, "can't convert String into Integer")
        end
      end

      describe 'flags を指定:' do
        it '正常' do
          Senna::Index.create("#{@tmpdir}/hoge", :flags=>Senna::Index::SPLIT_ALPHA).should be_kind_of(Senna::Index)
        end
      end

      describe 'flags に Integer 以外を指定:' do
        it 'TypeError' do
          proc{Senna::Index.create("#{@tmpdir}/hoge", :flags=>"123")}.should raise_error(TypeError, "can't convert String into Integer")
        end
      end

      describe 'initial_n_segments を指定:' do
        it '正常' do
          Senna::Index.create("#{@tmpdir}/hoge", :initial_n_segments=>100).should be_kind_of(Senna::Index)
        end
      end

      describe 'initial_n_segments に Integer 以外を指定:' do
        it 'TypeError' do
          proc{Senna::Index.create("#{@tmpdir}/hoge", :initial_n_segments=>"123")}.should raise_error(TypeError, "can't convert String into Integer")
        end
      end

      describe 'encoding を指定:' do
        it '正常' do
          Senna::Index.create("#{@tmpdir}/hoge", :encoding=>Senna::ENC_UTF8).should be_kind_of(Senna::Index)
        end
      end

      describe 'encoding に Integer 以外を指定:' do
        it 'TypeError' do
          proc{Senna::Index.create("#{@tmpdir}/hoge", :encoding=>"utf-8")}.should raise_error(TypeError, "can't convert String into Integer")
        end
      end
    end

    describe '.open:' do
      describe '存在するインデックスを指定:' do
        it 'Senna::Index オブジェクトを返す' do
          Senna::Index.create "#{@tmpdir}/hoge"
          Senna::Index.open("#{@tmpdir}/hoge").should be_kind_of(Senna::Index)
        end
      end

      describe '存在しないインデックスを指定:' do
        it 'Senna::Error' do
          proc{Senna::Index.open "#{@tmpdir}/hoge"}.should raise_error(Senna::Error)
        end
      end
    end

    describe '.remove:' do
      describe '存在するインデックスを指定:' do
        it '成功' do
          Senna::Index.create "#{@tmpdir}/hoge"
          proc{Senna::Index.remove("#{@tmpdir}/hoge")}.should_not raise_error
        end
      end

      describe '存在しないインデックスを指定:' do
        it 'Senna::Error' do
          proc{Senna::Index.remove("#{@tmpdir}/hoge")}.should raise_error(Senna::Error, "cannot remove #{@tmpdir}/hoge: No such file or directory")
        end
      end
    end

    describe '.rename:' do
      describe '存在するインデックスを指定:' do
        it '成功' do
          Senna::Index.create "#{@tmpdir}/hoge"
          proc{Senna::Index.rename "#{@tmpdir}/hoge", "#{@tmpdir}/fuga"}.should_not raise_error
        end
      end

      describe '存在しないインデックスを指定:' do
        it 'Senna::Error' do
          proc{Senna::Index.rename "#{@tmpdir}/hoge", "#{@tmpdir}/fuga"}.should raise_error(Senna::FileOperationError, "cannot rename #{@tmpdir}/hoge to #{@tmpdir}/fuga")
        end
      end
    end

    describe '#close:' do
      before do
        @idx = Senna::Index.create "#{@tmpdir}/hoge"
      end
      describe '未closeのインデックス:' do
        it '成功' do
          proc{@idx.close}.should_not raise_error
        end
      end

      describe 'close済みのインデックス:' do
        it 'RuntimeError' do
          @idx.close
          proc{@idx.close}.should raise_error(RuntimeError , 'already closed')
        end
      end
    end

    describe '#update & #select:' do
      before do
        @idx = Senna::Index.create "#{@tmpdir}/hoge", :flags=>Senna::Index::NGRAM
        @idx.update "123", nil, "pre record"
      end
      describe '新規登録:' do
        it 'レコードが２つ' do
          @idx.update "456", nil, "new record"
          @idx.select("record").nhits.should == 2
        end
      end

      describe '削除:' do
        it 'レコードが0' do
          @idx.update "123", "pre record", nil
          @idx.select("record").nhits.should == 0
        end
      end

      describe '更新:' do
        before do
          @idx.update "123", "pre record", "new record"
        end
        it '古い内容で検索されない' do
          @idx.select("pre").nhits.should == 0
        end
        it '新しい内容で検索される' do
          @idx.select("new").nhits.should == 1
        end
      end
    end

    describe '#select:' do
      before do
        @idx = Senna::Index.create "#{@tmpdir}/hoge", :flags=>Senna::Index::NGRAM
        @idx.update "123", nil, "abc def ghi jkl"
        @idx.update "456", nil, "def hoge fuga"
      end
      describe 'records 省略' do
        it 'Senna::Records が生成される' do
          recs = @idx.select "abc"
          recs.nhits.should == 1
        end
      end
      describe 'records 指定' do
        it 'records を返す' do
          recs = Senna::Records.new
          ret = @idx.select "abc", recs
          recs.nhits.should == 1
          ret.should == recs
        end
      end
      describe 'Senna::SEL_OR 指定' do
        it 'OR結果を返す' do
          recs = @idx.select "abc"
          recs.nhits.should == 1
          @idx.select "hoge", recs, Senna::SEL_OR
          recs.nhits.should == 2
        end
      end
      describe 'Senna::SEL_AND 指定' do
        it 'AND結果を返す' do
          recs = @idx.select "def"
          recs.nhits.should == 2
          @idx.select "hoge", recs, Senna::SEL_AND
          recs.nhits.should == 1
        end
      end
      describe 'Senna::SEL_BUT 指定' do
        it 'BUT結果を返す' do
          recs = @idx.select "def"
          recs.nhits.should == 2
          @idx.select "hoge", recs, Senna::SEL_BUT
          recs.nhits.should == 1
        end
      end
    end

    describe '#info' do
      before do
        @idx = Senna::Index.create "#{@tmpdir}/hoge", :key_size=>128, :flags=>Senna::Index::NGRAM|Senna::Index::SPLIT_ALPHA, :initial_n_segments=>10, :encoding=>Senna::ENC_UTF8
      end
      it 'create 時の情報を Hash で返す' do
        @idx.info.should == {
          :key_size => 128,
          :flags => Senna::Index::NGRAM|Senna::Index::SPLIT_ALPHA,
          :initial_n_segments => 10,
          :encoding => Senna::ENC_UTF8,
        }
      end
    end

  end

  describe Senna::Records do
    describe '検索結果:' do
      before do
        idx = Senna::Index.create "#{@tmpdir}/hoge", :flags=>Senna::Index::NGRAM
        idx.update "1", nil, "aaa abcdefg"
        idx.update "2", nil, "aaa 1234567"
        idx.update "3", nil, "aaa あいうえお"
        @rec = idx.select "a"
      end
      it '#each でキーを取り出せる' do
        ret = []
        @rec.each do |key, score|
          ret << [key, score]
        end
        ret.should == [["1", 2], ["2", 2], ["3", 2]]
      end
      it '#next で結果を読み込める' do
        @rec.next.should == ["1", 2]
        @rec.next.should == ["2", 2]
        @rec.next.should == ["3", 2]
        @rec.next.should be_nil
      end
      it '#rewind で最初に戻る' do
        @rec.next.should == ["1", 2]
        @rec.next.should == ["2", 2]
        @rec.rewind
        @rec.next.should == ["1", 2]
      end
      it '#curr_score でスコア値を返す' do
        @rec.next
        @rec.curr_score.should == 2
      end
      it 'カレントレコードがない状態で #curr_score は 0 を返す' do
        @rec.curr_score.should == 0
      end
      it '最後のレコードで #curr_score は 0 を返す' do
        @rec.next
        @rec.next
        @rec.next
        @rec.next
        @rec.curr_score.should == 0
      end
    end
  end

  describe '段落付き文書:' do
    before do
      @idx = Senna::Index.create "#{@tmpdir}/hoge", :flags=>Senna::Index::NGRAM
      @idx.update "key1", nil, "abc def ghi jkl", :section=>1
      @idx.update "key1", nil, "012 345 678 901", :section=>2
      @idx.update "key2", nil, "012 345 678 901", :section=>1
      @idx.update "key2", nil, "abc def ghi jkl", :section=>2
      @idx.update "key3", nil, "abc def ghi jkl"
    end
    it 'section 指定なしで全段落対象' do
      recs = @idx.select "abc"
      recs.next.first.should == "key1"
      recs.next.first.should == "key2"
      recs.next.first.should == "key3"
      recs.next.should == nil
    end
    it 'section 指定で指定段落のみ対象' do
      recs = @idx.select "abc", :weight_vector=>[1, 0]
      recs.next.first.should == "key1"
      recs.next.first.should == "key3"
      recs.next.should == nil
      recs = @idx.select "abc", :weight_vector=>[0, 1]
      recs.next.first.should == "key2"
      recs.next.should == nil
      recs = @idx.select "abc", :weight_vector=>[1, 1]
      recs.next.first.should == "key1"
      recs.next.first.should == "key2"
      recs.next.first.should == "key3"
      recs.next.should == nil
    end
  end
end

